<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\Manager;
use \Exception as Exception;
use \Teknisa\Libs\Utilities\Environment;
use Zeedhi\Framework\DataSource\FilterCriteria; 

class CadTipoLoja extends CrudWithParameterBag {
    
    const dataSourceTipoLoja      = 'ger12500_cadTipoLoja';
    const dataSourceLojasPorTipo  = 'ger12500_cadTipoLoja_relations';
    const dataSourceLojasPorTipoPerfil  = 'ger12500_cadTipoLoja_relations_perfil';
    protected $parameterBagColumns = array('NRORG', 'CDFILIAL');
    protected $parameterBagColumnsTipoLoja = array('NRORG', 'CDTIPOLOJA');
    protected $parameterBagColumnsTipoLojaPerfil = array('NRORG', 'CDTIPOLOJA', 'PERFIL');
    protected $util;
    protected $crudService;
    
              
    public function __construct (ParameterBag $parameterBag, Manager $dataSourceManager, $cadTipoLojaService, $util, $crudService, $environment) {
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->cadTipoService = $cadTipoLojaService;
        $this->util = $util;
		$this->crudService = $crudService;
		$this->environment = $environment;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceTipoLoja;
        $this->parameterBagColumns = $this->parameterBagColumns;
        
        parent::find($request, $response);     
    }
    
    public function findLojasPorTipo(DTO\Request\Filter $request, DTO\Response $response){
        if($this->environment->getIdUtlPerfilFilial() == 'S'){
            $conditions = $this->util->getParamsAsArray($request);
            
            $conditions['PERFIL'] = $this->environment->getNrPerfil();
            
            $dataSourceFilter = array();
            
            foreach($conditions as $key => $value) {
                $dataSourceFilter[] = array(
                    "columnName" => $key,
                    "operator"=> "=",
                    "value"=> $value
                );
            }
            
            $this->dataSourceName      = self::dataSourceLojasPorTipoPerfil;
            $this->parameterBagColumns = $this->parameterBagColumnsTipoLojaPerfil;
            
            $newRequest = new DTO\Request\Filter(
                new FilterCriteria(
                    $this->dataSourceName,
                    $dataSourceFilter,
                    $request->getFilterCriteria()->getPage(),
                    $request->getFilterCriteria()->getPageSize()
                ),
                $request->getMethod(),
                $request->getRoutePath(),
                $request->getUserId()
            );
            $request = $newRequest;
        } else {
            $this->dataSourceName      = self::dataSourceLojasPorTipo;
            $this->parameterBagColumns = $this->parameterBagColumnsTipoLoja;
        }
        parent::find($request, $response);     
    }
   
    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName    = self::dataSourceTipoLoja;
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        $this->validateRows($rows);
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
            
        } catch (Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteTipo(DTO\Request\Row $request, DTO\Response $response) {
        $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
        $this->dataSourceName    = self::dataSourceTipoLoja;
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            
            //Deleta Filhos
            $this->cadTipoService->deleteCascade($dataSet->getRows()[0]);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function deleteLojaPorTipo(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName    = self::dataSourceLojasPorTipo;
        
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function saveLojaPorTipo(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName    = self::dataSourceLojasPorTipo;
        
        parent::save($request, $response);
    }
    
    protected function validateRows(array $rows) {  }
}
