<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Utilities\Util;
use Teknisa\MAN\Services\FileUploaderService;

class Observacao extends CrudWithParameterBag {
    
    protected $parameterBag;
    protected $dataSourceNameArray = array (
        'filterOcorrencia'        => 'vnd57000_cadastrodeobs_ocorrencias',
        'filterGrupos'            => 'vnd57000_cadastrodeobs_grupos',
        'filterProdutos'          => 'vnd57000_cadastrodeobs_produtos',
        'filterProdutosProds'     => 'vnd57000_cadastrodeobs_produtos_produtos',
        'filterObsProduto'        => 'vnd57000_cadastrodeobs_obsproduto',
        'filterEachObsProduto'    => 'vnd57000_cadastrodeobs_obsproduto_produtos',
        'filterGrpObrig'          => "vnd57000_cadastrodeobs_grupObrig",
        "filterProdObrigOcor"     => "vnd57000_cadastrodeobs_prodObrig",
        "filterOcorrenciaObrig"   => 'vnd57000_cadastrodeobs_ocorrencias_obrig'
    );
    protected $parameterBagColumnsArray = array (
        'filterOcorrencia'        =>  array('NRORG', 'CDGRPOCOR'),
        'filterGrupos'            =>  array('NRORG'),
        'filterProdutos'          =>  array('NRORG', 'CDGRPOCOR', "CDOCORR"),
        'filterEachProduto'       =>  array('NRORG', 'CDPRODUTO', 'CDGRPOCOR'),
        'filterGrpObrig'          =>  array("CDGRUPOOBRIG","CDGRPOCOR","NRORG"),
        "filterProdObrigOcor"     =>  array("CDGRUPOOBRIG","CDGRPOCOR","NRORG","CDOCORR"),
        'filterOcorrenciaObrig'   =>  array("CDGRUPOOBRIG","CDGRPOCOR","NRORG","CDOCORR")
    );
    
    protected $observacaoService;
    protected $fileUploaderService;

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, $observacaoService, FileUploaderService $fileUploaderService) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->observacaoService = $observacaoService;
        $this->fileUploaderService = $fileUploaderService;
    }
    
    public function saveDefaultObservacao ($row, $response, $dataSourceNameSave) {
        try {
            $dataSet = new DataSet($dataSourceNameSave, array($row));
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($dataSourceNameSave, $persistedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }    
    }
    
    public function deleteDefaultObservacao ($request, $response, $dataSourceNameDelete, $isProduto = false, $isObsProduto = false) {
        try {
            if (!$isProduto)
                $row = array($request->getRow());
            else
                $row = $request;
            $dataSet = new DataSet($dataSourceNameDelete, $row);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet($dataSet);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findOcorrencia (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterOcorrencia'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterOcorrencia'];
        parent::find($request, $response);
    }
    
    public function saveOcorrencia (DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        
        if(count($rows) === 1) {
            $row = $rows[0];
            $this->fileUploaderService->setImageForSave($row, 'DSENDEIMGOCORR', 'DSENDEIMGOCORR');
            self::saveDefaultObservacao($row, $response, $this->dataSourceNameArray['filterOcorrencia']);
        }
    }
    
    public function deleteOcorrencia (DTO\Request\Row $request, DTO\Response $response) {
        self::deleteDefaultObservacao($request, $response, $this->dataSourceNameArray['filterOcorrencia']);
    }
    
    public function findGrupos (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterGrupos'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterGrupos'];
        parent::find($request, $response);
    }
    
    public function saveGrupos (DTO\Request\DataSet $request, DTO\Response $response) {
        self::saveDefaultObservacao($request->getDataSet()->getRows()[0], $response, $this->dataSourceNameArray['filterGrupos']);
    }
    
    public function deleteGrupos (DTO\Request\Row $request, DTO\Response $response) {
        self::deleteDefaultObservacao($request, $response, $this->dataSourceNameArray['filterGrupos']);
    }  
    
    public function findProdutos (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterProdutos'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterProdutos'];
        parent::find($request, $response);
    }
    
    public function findObsProduto (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterObsProduto'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterOcorrencia'];
        parent::find($request, $response);
    }
    
    public function findEachObsProduto (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterEachObsProduto'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterEachProduto'];
        parent::find($request, $response);
    }
    
    public function saveProdutos (DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $CDARVPRODINI   = $row['CDARVPRODINI'];
        $CDARVPRODFIM   = $row['CDARVPRODFIM'];
        $PRODUTOS       = $row['PRODUTOS'];
        $CDGRPOCOR      = $row['CDGRPOCOR'];
        $CDOCORR        = $row['CDOCORR'];
        $CDOPERINCLUSAO = $row['CDOPERINCLUSAO'];
        $CDNVPRODUTO    = $row['CDNVPRODUTO'];
        $CDOPERULTATU   = $row['CDOPERULTATU'];
        $NRORGULTATU    = $row['NRORGULTATU'];
        $NRORGINCLUSAO  = $row['NRORGINCLUSAO'];
        $DTINCLUSAO     = $row['DTINCLUSAO'];
        $DTULTATUPOCO   = $row['DTULTATU'];
        $DTULTATU       = $row['DTULTATU'];
        $NRORG          = $row['NRORG'];
        $save = $this->observacaoService->saveProdutos($CDARVPRODINI, $CDARVPRODFIM, $CDGRPOCOR, $CDOCORR, $CDOPERINCLUSAO, $CDOPERULTATU,
            $NRORGULTATU, $NRORGINCLUSAO, $DTINCLUSAO, $DTULTATUPOCO, $DTULTATU, $NRORG, $PRODUTOS, $CDNVPRODUTO) ;
        if(!$save){
           $response->addMessage(new Message("Não existe produtos no intervalo selecionado!",Message::TYPE_WARNING));
        }
    }
    
    public function deleteProdutos (DTO\Request\DataSet $request, DTO\Response $response) {
        self::deleteDefaultObservacao($request->getDataSet()->getRows(), $response, $this->dataSourceNameArray['filterProdutos'], true);
    } 
    
    public function findProdutosProds (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterProdutosProds'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterProdutos'];
        parent::find($request, $response);
    }
    
    public function deleteMultiplo ($request, $response){
        try {
            $dataSet = $request->getDataSet();
            $this->dataSourceName = 'vnd57000_cadastrodeobs_ocorrencias';
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
            $nrDeletedRows = count($deletedRows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function saveObsProdInLine(DTO\Request $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'vnd57000_cadastrodeobs_obsproduto';
            $rows = $request->getDataSet()->getRows();
            foreach($rows as $product){
                self::save($request,$response);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deleteObsProduto(DTO\Request\DataSet $request, DTO\Response $response) {
        $request = $request->getDataSet()->getRows();
        foreach($request as $row){
            try {
                $CDGRPOCOR  = $row["CDGRPOCOR"];
                $CDOCORR    = $row["CDOCORR"];
                $CDPRODUTO  = $row["CDPRODUTO"];
                $NRORG      = $row["NRORG"];
                $this->observacaoService->deleteObsProduto($CDGRPOCOR, $CDOCORR, $CDPRODUTO, $NRORG);
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
            }
        }
    }
    
    private function setParametersDataSource ($param){
        $this->dataSourceName = $this->dataSourceNameArray[$param];
        $this->parameterBagColumns = $this->parameterBagColumnsArray[$param];
    }
    
    public function findGrupoObrigatorio (DTO\Request\Filter $request, DTO\Response $response){
         $this->setParametersDataSource('filterGrpObrig');
         parent::find($request, $response);
    }
    
    public function saveGruposObrig (DTO\Request\DataSet $request, DTO\Response $response){
        $this->setParametersDataSource('filterGrpObrig');
        $dataSet = $request->getDataSet();
        $row = $request->getDataSet()->getRows()[0];
        
        if($row['__is_new']){
            $request = $this->observacaoService->newRequest($request);
            parent::save($request, $response);
        } else {
            $params = array(
                ':NMGRUPOBRIG'  => $row['NMGRUPOBRIG'],
                ':CDGRPOCOR'    => $row['CDGRPOCOR'],
                ':CDGRUPOOBRIG' => $row['CDGRUPOOBRIG']
            );
            
            $this->observacaoService->updateGrupoObs($params);
        }
    }
    
    public function removeGruposObrig (DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $deletedRows = $this->observacaoService->deleteGrupObrig($request->getDataSet()->getRows());
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
            $nrDeletedRows = count($deletedRows);
            $response->addNotification(new Notification("{$nrDeletedRows} row(s) deleted.", Notification::TYPE_SUCCESS));
        }catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString())); 
        }
    }
    
     public function findProdObrigOcor (DTO\Request\Filter $request, DTO\Response $response){
         $this->setParametersDataSource('filterProdObrigOcor');
         parent::find($request, $response);
    }
    
    public function saveProdObrigOcor(DTO\Request\Row $request, DTO\Response $response){
        try{
            $this->observacaoService->saveProdObrigOcor($request->getRow());
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findOcorrenciaObrig (DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterOcorrenciaObrig'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterOcorrenciaObrig'];
        parent::find($request, $response);
    }
    
    public function removeProdObrig (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $deletedRows = $this->observacaoService->deleteProdObrig($request->getDataSet()->getRows());
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
            $nrDeletedRows = count($deletedRows);
            $response->addNotification(new Notification("{$nrDeletedRows} row(s) deleted.", Notification::TYPE_SUCCESS));
        }catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString())); 
        }
    }
    
    public function saveOservacaoProdutos (DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = $this->dataSourceNameArray['filterEachObsProduto'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterEachProduto'];
        parent::save($request,$response);
    }
}    