<?php

namespace Teknisa\MAN\Factories;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;

class ConsumoRealizadoFactory {

    protected $connection;
    protected $nrOrg;

    public function __construct(EntityManager $entityManager, Environment $environment) {
        $this->entityManager    = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->nrOrg = $environment->getNrOrg();
        $this->cdOperador = $environment->getCdOperador();
    }

    public function findConsumoRealizado($params) {
        try{
            //$params["NRORG"] = $this->nrOrg;
            $sql = "SELECT C.NRCONSREAL,
                        C.CDFILIAL,
                        F.NMFILIAL,
                        C.DTCONSUMO,
                        C.CDPRODUTO,
                        P.CDARVPROD,
                        P.NMPRODUTO,
                        C.QTCONSUMO,
                        C.VRCONSUMO,
                        C.VRTOTCONSUMO,
                        C.NRLANCESTQ,
                        DECODE(C.NRLANCESTQ, NULL, 'N', 'S') AS RETIRADAREALIZADA
                            FROM CONSUMOREAL C, FILIAL F, PRODUTO P
                                WHERE 
                                (C.CDFILIAL = :CDFILIAL) AND
                                (C.CDFILIAL = F.CDFILIAL) AND
                                (C.CDPRODUTO BETWEEN NVL(:CDPRODUTOINI, C.CDPRODUTO) AND NVL(:CDPRODUTOFIN, C.CDPRODUTO)) AND
                                (C.CDPRODUTO = P.CDPRODUTO ) AND
                                (C.DTCONSUMO BETWEEN :DTINICIAL AND :DTFINAL) AND
                                (DECODE(C.NRLANCESTQ, NULL, 'N', 'S') = NVL(:NRLANCESTQ, DECODE(C.NRLANCESTQ, NULL, 'N', 'S'))) AND
                                ((DECODE(C.NRLANCESTQ, NULL, 'N', 'S') = :RETIRADAREALIZADA) OR (:RETIRADAREALIZADA IS NULL))
                                ORDER BY C.NRCONSREAL,
                                    C.DTCONSUMO,
                                    C.CDPRODUTO";
            $newParams = array(
                'CDFILIAL'      =>$params['CDFILIAL'],
                'CDPRODUTOINI'  =>$params['CDPRODUTOINI'],
                'CDPRODUTOFIN'  =>$params['CDPRODUTOFIN'],
                'DTINICIAL'     =>$params['DTINICIAL'],
                'DTFINAL'       =>$params['DTFINAL'],
                'NRLANCESTQ'   =>isset($params['NRLANCESTQ']) ? $params['NRLANCESTQ'] : null,
                'RETIRADAREALIZADA' => isset($params['RETIRADAREALIZADA']) ? $params['RETIRADAREALIZADA'] : null
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($newParams);
            return $std->fetchAll();
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function insertConsumoRealizado($row){
        try{
            
            $sql = "INSERT INTO CONSUMOREAL (NRCONSREAL, CDFILIAL, DTCONSUMO, CDPRODUTO, QTCONSUMO, VRCONSUMO, VRTOTCONSUMO)
                        VALUES (:NRCONSREAL,
                                :CDFILIAL,
                                :DTCONSUMO,
                                :CDPRODUTO,
                                :QTCONSUMO,
                                :VRCONSUMO,
                                :VRTOTCONSUMO)";
            
            $params = array(
                'NRCONSREAL'=>$row['NRCONSREAL'],
                'CDFILIAL'=>$row['CDFILIAL'],
                'DTCONSUMO'=>$row['DTCONSUMO'],
                'CDPRODUTO'=>$row['CDPRODUTO'],
                'QTCONSUMO'=>$row['QTCONSUMO'],
                'VRCONSUMO'=>$row['VRCONSUMO'],
                'VRTOTCONSUMO'=>$row['VRTOTCONSUMO']
            );
            
            $std = $this->connection->prepare($sql);
            $result = $std->execute($params);
            
            return $result;
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function updateConsumoRealizado($row){
        try{
            $sql = "UPDATE CONSUMOREAL 
                    SET DTCONSUMO = :DTCONSUMO,
                    CDPRODUTO = :CDPRODUTO,
                    QTCONSUMO = :QTCONSUMO, 
                    VRCONSUMO = :VRCONSUMO, 
                    VRTOTCONSUMO = :VRTOTCONSUMO
                        WHERE   (CDFILIAL = :CDFILIAL) AND
                                (NRCONSREAL = :NRCONSREAL)";
            
            $params = array(
                'CDFILIAL'=>$row['CDFILIAL'],
                'NRCONSREAL'=>$row['NRCONSREAL'],
                'DTCONSUMO'=>$row['DTCONSUMO'],
                'CDPRODUTO'=>$row['CDPRODUTO'],
                'QTCONSUMO'=>$row['QTCONSUMO'],
                'VRCONSUMO'=>$row['VRCONSUMO'],
                'VRTOTCONSUMO'=>$row['VRTOTCONSUMO']
            );
            
            $std = $this->connection->prepare($sql);
            $update = $std->execute($params);
            return $update;
            
        }catch (\Exception $e) {
            throw $e;
        }
    } 
    
    public function deleteConsumoRealizado($row){
        try{
            $sql = "DELETE FROM CONSUMOREAL 
                        WHERE   CDFILIAL = :CDFILIAL AND
                                DTCONSUMO = :DTCONSUMO AND
                                CDPRODUTO = :CDPRODUTO ";
            
            $params = array(
                'CDFILIAL'=>$row['CDFILIAL'],
                'DTCONSUMO'=>$row['DTCONSUMO'],
                'CDPRODUTO'=>$row['CDPRODUTO']
            );
            
            $std = $this->connection->prepare($sql);
            $result = $std->execute($params);
            return $result;
            
        }catch (\Exception $e) {
            throw $e;
        }
    } 
    
    public function verificaConsumoRealizado($row){
        try{
            $sql = "SELECT * FROM CONSUMOREAL 
                        WHERE  CDFILIAL = :CDFILIAL AND
                               DTCONSUMO = :DTCONSUMO AND
                               CDPRODUTO = :CDPRODUTO AND
                               DECODE(NRLANCESTQ, NULL, 'N', 'S') = 'N'";
            
            $params = array(
                'CDFILIAL'=>$row['CDFILIAL'],
                'DTCONSUMO'=>$row['DTCONSUMO'],
                'CDPRODUTO'=>$row['CDPRODUTO']
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetch();
            
        }catch (\Exception $e) {
            throw $e;
        }
    } 
    
    public function getProdutoAcabadoProdutosComposicao($row){
        try{
            $sql = "SELECT P.CDPRODUTO,
                        P.CDARVPROD,
                        R.QTSUBPRODROT AS QTITEMCOMPOS,
                        R.NRHRFABPROD,
                        R.NRSEQPROD AS NRSEQUE,
                        R.IDIMPPRODROT,
                        P.NMPRODUTO,
                        P.CDPRODESTO,
                        P.SGUNIDADE,
                        P.VRFATOCONV,
                        PF.CDFILIAL,
                        PF.IDCNTRESTOQ,
                        R.NRORG,
                        'COMPOSTO' AS TIPOPROD
                            FROM ROTFABRICA R,
                            PRODUTO P,
                            PRODUTO O,
                            (SELECT COUNT(*) AS NRCOMPPROD, CDPRODUTO FROM ROTFABRICA GROUP BY CDPRODUTO
                            ) F,
                            PRODFILI PF
                                WHERE (O.CDPRODUTO  = :CDPRODUTO)
                                AND (O.CDPRODUTO    = R.CDPRODUTO)
                                AND (R.CDSUBPRODUTO = P.CDPRODUTO)
                                AND (F.CDPRODUTO(+) = R.CDSUBPRODUTO)
                                AND (PF.CDFILIAL    = :CDFILIAL)
                                AND (PF.CDPRODUTO   = P.CDPRODUTO)
                    UNION 
                    SELECT  P.CDPRODUTO,
                            P.CDARVPROD,
                            1 AS QTITEMCOMPOS,
                            NULL as NRHRFABPROD,
                            NULL AS NRSEQUE,
                            NULL AS IDIMPPRODROT,
                            P.NMPRODUTO,
                            P.CDPRODESTO,
                            P.SGUNIDADE,
                            P.VRFATOCONV,
                            PF.CDFILIAL,
                            PF.IDCNTRESTOQ,
                            P.NRORG,
                            'ESTOQUE' AS TIPOPROD
                    FROM PRODUTO P,
                    PRODFILI PF
                    WHERE (P.CDPRODUTO  = :CDPRODUTO)
                    AND (PF.CDFILIAL    = :CDFILIAL)
                    AND (PF.CDPRODUTO   = P.CDPRODUTO)";
            
            $params = array(
                'CDPRODUTO'=>$row['CDPRODUTO'],
                'CDFILIAL'=>$row['CDFILIAL']
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        }catch (\Exception $e) {
            throw $e;
        }
    } 
    
    public function adqinsertlancto($row){
        try{
            $sql = "INSERT INTO LANCTOESTOQ
                        (CDFILIAL, NRLANCESTQ, IDTPLANCTO, DTLANCESTQ, DSLANCESTQ,
                        CDFILIMOVI, CDOPERADOR)
                        VALUES
                        (:CDFILIAL, :NRLANCESTQ, :IDTPLANCTO, :DTLANCESTQ, :DSLANCESTQ,
                        :CDFILIMOVI, :CDOPERADOR)";
            
            $params = array(
                       "CDFILIAL"   => $row["CDFILIAL"],
                       "NRLANCESTQ" => $row['NRLANESTSAID'],
                       "IDTPLANCTO" => $row["IDTPLANCTO"],
                       "DTLANCESTQ" => $row["DTLANCESTQ"],
                       "DSLANCESTQ" => $row['DSLANCESTQ'],
                       "CDFILIMOVI" => $row["CDFILIAL"],
                       "CDOPERADOR" => $this->cdOperador
                    );
            
            $std = $this->connection->prepare($sql);
            $result = $std->execute($params);
            return $result;
            
        } catch (\Exception $e) {
            throw $e;
        }
    } 
    
    public function adqinsertretiradaesto($row){
        try{
            $sql = "INSERT INTO RETIRADAESTO
                      (CDFILIAL,   NRRETEST, DTRETEST,   DSRETEST,  CDFILIALLAN,
                       NRLANCESTQ, CDSETOR,  CDCENTCUST)
                    VALUES
                      (:CDFILIAL,   :NRRETEST, :DTRETEST,   :DSRETEST,  :CDFILIALLAN,
                       :NRLANCESTQ, :CDSETOR,  :CDCENTCUST)";
            
            $params = array(
                       "CDFILIAL"       => $row["CDFILIAL"],
                       "NRRETEST"       => $row['NRRETEST'],
                       "DTRETEST"       => $row["DTLANCESTQ"],
                       "DSRETEST"       => $row["DSLANCESTQ"],
                       "CDFILIALLAN"    => $row['CDFILIAL'],
                       "NRLANCESTQ"     => $row["NRLANESTSAID"],
                       "CDSETOR"        => $row["CDSETOR"],
                       "CDCENTCUST"     => $row["CDCENTCUST"]
                    );
            
            $std = $this->connection->prepare($sql);
            $result = $std->execute($params);
            return $result;
            
        }catch (\Exception $e) {
            throw $e;
        }
    } 
    
    public function getnrseqitem($row){
        try{
            $sql = "SELECT MAX(NRSEQUITEM) FROM ITLANCTOEST 
                        WHERE  CDFILIAL = :CDFILIAL AND
                               NRLANCESTQ = :NRLANCESTQ";
            
            $params = array(
                'CDFILIAL'=>$row['CDFILIAL'],
                'NRLANCESTQ'=>$row['NRLANESTSAID']
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetch();
            
        } catch (\Exception $e) {
            throw $e;
        }
    } 
    
    public function adqmaxdtposest($subProduto){
        try{
            $sql = "SELECT TO_CHAR(NVL(MAX(DTPOSIESTQ), SYSDATE), 'DD/MM/YYYY') DTPOSIESTQ
                        FROM POSESTDIA
                        WHERE (CDFILIAL = :CDFILIAL)
                          AND (CDALMOXARIFE = :CDALMOXARIFE)
                          AND (CDLOCALESTOQ = :CDLOCALESTOQ)
                          AND (NRLOTEESTQ = :NRLOTEESTQ)
                          AND (NRSUBLOTE = :NRSUBLOTE)
                          AND (CDPRODUTO = :CDPRODESTO)
                          AND (DTPOSIESTQ <= :DTPOSICAO)";
            
            $params = array(
                'CDFILIAL'=>$subProduto['CDFILIAL'],
                'CDALMOXARIFE'=>isset($subProduto["CDALMOXARIFE"]) ? $subProduto["CDALMOXARIFE"] : " ",
                'CDLOCALESTOQ'=>isset($subProduto["CDLOCALESTOQ"]) ? $subProduto["CDLOCALESTOQ"] : " ",
                'NRLOTEESTQ'=>isset($subProduto["NRLOTEESTQ"]) ? $subProduto["NRLOTEESTQ"] : " ",
                'NRSUBLOTE'=>isset($subProduto["NRLOTEESTQ"]) ? $subProduto["NRLOTEESTQ"] : " ",
                'CDPRODESTO'=>$subProduto['CDPRODUTO'],
                'DTPOSICAO'=>isset($subProduto['DTPOSICAO']) ? $subProduto['DTPOSICAO'] : NULL
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        }catch (\Exception $e) {
            throw $e;
        }
    } 
  
    public function adqposestlanc($subProduto, $retirada){
        try{
            $sql = "SELECT P.CDPRODUTO,  P.NMPRODUTO,  P.SGUNIDADE, IDCNTRESTOQ,
                        P.VRFATOCONV, (D.QTESTOQDIA/P.VRFATOCONV) + NVL((QTLANCTOEST/P.VRFATOCONV),0) AS QTESTOQDIA, P.CDPRODESTO,
                        NVL(CUSTO_PRODUTO_LCEST(:CDFILIAL, D.CDPRODUTO, TO_NUMBER(NVL(CDTIPCUSTEST, '3')), :DTCUSTO, SYSDATE,
                        D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE),0) * P.VRFATOCONV AS VRCUSTOPROD,
                        NVL(CUSTO_PRODUTO_LCEST(:CDFILIAL, D.CDPRODUTO, 4, :DTCUSTO, SYSDATE,
                        D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE),0) * P.VRFATOCONV AS VRCUSTOBRUT
                        FROM PRODUTO P, POSESTDIA D, PRODFILI F, PARAMFILIAL PF,
                             (SELECT QTLANCTOEST, CDPRODUTO
                              FROM ITLANCTOEST
                              WHERE CDPRODMOVI   = :CDPRODUTO   AND
                                    CDFILIMOVI  = :CDFILIAL    AND
                                    CDFILIAL    = :CDFILIPROC  AND
                                    NRLANCESTQ  = :NRLANCESTQ) I
                        
                        WHERE (P.CDPRODUTO    = :CDPRODUTO)    AND
                              (D.CDFILIAL     = :CDFILIAL)     AND
                              (D.CDALMOXARIFE = :CDALMOXARIFE) AND
                              (D.CDLOCALESTOQ = :CDLOCALESTOQ) AND
                              (D.NRLOTEESTQ   = :NRLOTEESTQ)   AND
                              (D.NRSUBLOTE    = :NRSUBLOTE)    AND
                              (D.CDPRODUTO    = P.CDPRODESTO)    AND
                              (D.CDPRODUTO    = I.CDPRODUTO(+))  AND
                              (P.CDPRODESTO   = F.CDPRODUTO)     AND
                              (F.CDFILIAL     = D.CDFILIAL)      AND
                              (PF.CDFILIAL    = D.CDFILIAL)     AND
                              (DTPOSIESTQ     = :DTPOSICAO)";
            
            $params = array(
                'CDFILIAL'      =>$subProduto['CDFILIAL'],
                'DTCUSTO'       =>$subProduto['DTCUSTO'],
                'CDPRODUTO'     =>$subProduto['CDPRODUTO'],
                'CDFILIPROC'    =>$subProduto['CDFILIAL'],
                'NRLANCESTQ'    =>null,
                'CDALMOXARIFE'  =>isset($subProduto["CDALMOXARIFE"]) ? $subProduto["CDALMOXARIFE"] : ' ',
                'CDLOCALESTOQ'  =>isset($subProduto["CDLOCALESTOQ"]) ? $subProduto["CDLOCALESTOQ"] : ' ',
                'NRLOTEESTQ'    =>isset($subProduto["NRLOTEESTQ"]) ? $subProduto["NRLOTEESTQ"] : ' ',
                'NRSUBLOTE'     =>isset($subProduto["NRLOTEESTQ"]) ? $subProduto["NRLOTEESTQ"] : ' ',
                'DTPOSICAO'     =>$subProduto['DTPOSICAO']
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        }catch (\Exception $e) {
            throw $e;
        }
    } 
    
    public function selectcdprodmovi($subProduto){
        try{
            $sql = "SELECT CDPRODUTO, NMPRODUTO, CDNVPRODUTO, CDPRODESTO 
                    FROM PRODUTO 
                        WHERE CDPRODUTO BETWEEN :CDPRODUTOINI AND :CDPRODUTOFIN";
            
            $params = array(
                'CDPRODUTOINI'  =>substr($subProduto['CDPRODESTO'], 0, -2),
                'CDPRODUTOFIN'  =>substr($subProduto['CDPRODESTO'], 0, -2).'ZZ'
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        }catch (\Exception $e) {
            throw $e;
        }
    } 
    
    public function adqinsertitemlancesto($subProduto, $retirada){
        try{
            $sql = "INSERT INTO ITLANCTOEST
                      (NRORG, CDFILIAL, NRLANCESTQ, NRSEQUITEM, CDPRODUTO, DSLANCESTIT,
                        CDALMOXARIFE,CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE, CDFILIMOVI, CDPRODMOVI,
                        IDORDEMCLC, DTLANCMOVI, IDEVOLESTOQ, CDTIPORETI, CDCENTCUST, QTTOTLANCTO, IDTIPOMOVI, QTLANCTOEST, 
                        VRUNILANCTO, VRTOTLANCTO, QTDEVOLESTOQ, VRLANCTOEST , VRLANCTOBRUT, NRREQUESTO, NRSEQITEMREQEST)
                    VALUES
                      (:NRORG, :CDFILIAL, :NRLANCESTQ, :NRSEQUITEM, :CDPRODUTO, :DSLANCESTIT,
                        NVL(:CDALMOXARIFE, ' '),
                        NVL(:CDLOCALESTOQ,' '), 
                        NVL(:NRLOTEESTQ, ' '), 
                        NVL(:NRSUBLOTE, ' '), 
                        :CDFILIMOVI, :CDPRODMOVI, :IDORDEMCLC, :DTLANCMOVI, :IDEVOLESTOQ, :CDTIPORETI, 
                        :CDCENTCUST, :QTTOTLANCTO, :IDTIPOMOVI, :QTLANCTOEST, :VRUNILANCTO, :VRTOTLANCTO, 
                        :QTDEVOLESTOQ, :VRLANCTOEST , :VRLANCTOBRUT, :NRREQUESTO, :NRSEQITEMREQEST)";
            
            $params = array(
                       "NRORG"          => $this->nrOrg,
                       "CDFILIAL"       => $retirada["CDFILIAL"],
                       "NRLANCESTQ"     => $retirada['NRLANESTSAID'],
                       "NRSEQUITEM"     => $subProduto["NRSEQUITEM"],
                       "CDPRODUTO"      => $subProduto["CDPRODUTO"],
                       "DSLANCESTIT"    => $retirada["DSLANCESTIT"],
                       "CDALMOXARIFE"   => isset($subProduto["CDALMOXARIFE"]) ? $subProduto["CDALMOXARIFE"] : NULL,
                       "CDLOCALESTOQ"   => isset($subProduto["CDLOCALESTOQ"]) ? $subProduto["CDLOCALESTOQ"] : NULL,
                       "NRLOTEESTQ"     => isset($subProduto["NRLOTEESTQ"]) ? $subProduto["NRLOTEESTQ"] : NULL,
                       "NRSUBLOTE"      => isset($subProduto['NRSUBLOTE']) ? $subProduto['NRSUBLOTE'] : NULL,
                       "CDFILIMOVI"     => $retirada["CDFILIAL"],
                       "CDPRODMOVI"     => $subProduto['CDPRODMOVI'],
                       "IDORDEMCLC"     => '9',     //PADRAO ?? CONFIRMAR COM FABRICIO
                       "DTLANCMOVI"     => $retirada["DTLANCESTQ"],
                       "IDEVOLESTOQ"    => 'S',     
                       "CDTIPORETI"     => $retirada["CDTIPORETI"],
                       "CDCENTCUST"     => $retirada['CDCENTCUST'],
                       "QTTOTLANCTO"    => $subProduto["QTTOTLANCTO"],
                       "IDTIPOMOVI"     => '3',     //PADRAO ?? CONFIRMAR COM FABRICIO
                       "QTLANCTOEST"    => $subProduto['QTLANCTOEST'],
                       "VRUNILANCTO"    => $subProduto["VRUNILANCTO"],
                       "VRTOTLANCTO"    => $subProduto["VRTOTLANCTO"],
                       "QTDEVOLESTOQ"   => isset($subProduto["QTDEVOLESTOQ"]) ? $subProduto["QTDEVOLESTOQ"] : 0,
                       "VRLANCTOEST"    => $subProduto["VRLANCTOEST"],
                       "VRLANCTOBRUT"   => $subProduto["VRLANCTOBRUT"],
                       "NRREQUESTO"     => isset($subProduto["NRREQUESTO"]) ? $subProduto["NRREQUESTO"] : NULL,
                       "NRSEQITEMREQEST"=> isset($subProduto["NRSEQITEMREQEST"]) ? $subProduto["NRSEQITEMREQEST"] : NULL
                );
            
            $std = $this->connection->prepare($sql);
            $insert = $std->execute($params);
            
        }catch (\Exception $e) {
            throw $e;
        }
    } 
    
    public function updateStatusConsumoRealizado($row, $lanctoestoq){
        try{
            $sql = "UPDATE CONSUMOREAL 
                    SET NRLANCESTQ = :NRLANCESTQ
                        WHERE   CDFILIAL = :CDFILIAL AND
                                DTCONSUMO = :DTCONSUMO AND
                                CDPRODUTO = :CDPRODUTO AND
                                NRCONSREAL = :NRCONSREAL";
            
            $params = array(
                'CDFILIAL'=>$row['CDFILIAL'],
                'DTCONSUMO'=>$row['DTCONSUMO'],
                'CDPRODUTO'=>$row['CDPRODUTO'],
                'NRCONSREAL'=>$row['NRCONSREAL'],
                'NRLANCESTQ'=>$lanctoestoq
            );
            
            $std = $this->connection->prepare($sql);
            $update = $std->execute($params);
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function retornaDadosProcessFilial($filial, $organizacao) {
        try{
            //$params["NRORG"] = $this->nrOrg;
            $sql = "SELECT PF.CDFILIAL,
                        DECODE(PG.IDUTILDTPROC, 'S', DECODE(PF.DTPROCESSA, NULL, TO_DATE(SYSDATE), PF.DTPROCESSA), TO_DATE(SYSDATE)) AS DTPROCESSA,
                        PF.IDLANCDTMAIFIL,
                        PE.IDLANCDTMAI,
                        PF.IDLANCDTMENFIL,
                        PE.IDLANCDTMEN,
                        NVL(PF.NRDIASENTRFIL, 0)   AS NRDIASENTRFIL,
                        NVL(PE.NRDIASENTR, 0)      AS NRDIASENTR,
                        NVL(PF.NRDIASSAIDFIL, 0)   AS NRDIASSAIDFIL,
                        NVL(PE.NRDIASSAID, 0)      AS NRDIASSAID,
                        NVL(PF.NRDIASAJUSFIL, 0)   AS NRDIASAJUSFIL,
                        NVL(PE.NRDIASAJUS, 0)      AS NRDIASAJUS,
                        NVL(PF.NRDIASENTRPDFIL, 0) AS NRDIASENTRPDFIL,
                        NVL(PE.NRDIASENTRPD, 0)    AS NRDIASENTRPD,
                        NVL(PF.NRDIASSAIDPDFIL, 0) AS NRDIASSAIDPDFIL,
                        NVL(PE.NRDIASSAIDPD, 0)    AS NRDIASSAIDPD,
                        NVL(PF.NRDIASAJUSPDFIL, 0) AS NRDIASAJUSPDFIL,
                        NVL(PE.NRDIASAJUSPD, 0)    AS NRDIASAJUSPD,
                        PG.NRORG
                            FROM PARAMGERAL PG,
                            PARAMFILIAL PF,
                            PARAMESTOQUE PE
                                WHERE PG.NRORG = :NRORG
                                AND PF.NRORG = PG.NRORG
                                AND PE.NRORG   = PG.NRORG
                                AND PF.CDFILIAL = :CDFILIAL";
            $params = array(
                'CDFILIAL'  =>$filial, 
                'NRORG'     =>$organizacao
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetch();
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function buscaLanctoestoq($cdfilial, $nrlancestq, $nrorg) {
        try{
            
            $sql = "SELECT * FROM LANCTOESTOQ
                        WHERE CDFILIAL = :CDFILIAL
                            AND NRLANCESTQ = :NRLANCESTQ
                            AND NRORG = :NRORG";
            $params = array(
                'CDFILIAL'  =>$cdfilial, 
                'NRLANCESTQ'  =>$nrlancestq, 
                'NRORG'     =>$nrorg
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetch();
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function buscaItlanctoest($cdfilial, $nrlancestq, $nrorg, $idevolestoq) {
        try{
            //$params["NRORG"] = $this->nrOrg;
            $sql = "SELECT * FROM ITLANCTOEST
                        WHERE CDFILIAL = :CDFILIAL
                            AND  NRLANCESTQ = :NRLANCESTQ
                            AND NRORG = :NRORG
                            AND IDEVOLESTOQ = :IDEVOLESTOQ";
            $params = array(
                'CDFILIAL'  =>$cdfilial, 
                'NRLANCESTQ'  =>$nrlancestq, 
                'NRORG'     =>$nrorg,
                'IDEVOLESTOQ' =>$idevolestoq
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function selectMovi($nrmovilote, $nrorg) {
        try{
            //$params["NRORG"] = $this->nrOrg;
            $sql = "  SELECT * FROM MOVILOTE
                          WHERE NRMOVILOTE = :NRMOVILOTE 
                          AND NRORG = :NRORG";
            $params = array( 
                'NRMOVILOTE'  =>$nrmovilote, 
                'NRORG'     =>$nrorg
            );
            
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetch();
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function atualizaItensLancamentoEstoque($cdfilial, $nrlancestq, $nrOrg, $idevolestoq) {
        try{
            //$params["NRORG"] = $this->nrOrg;
            $sql = "UPDATE ITLANCTOEST
                        SET IDEVOLESTOQ = :IDEVOLESTOQ
                        WHERE CDFILIAL = :CDFILIAL
                            AND NRLANCESTQ = :NRLANCESTQ
                            AND NRORG = :NRORG";
            $params = array(
                'CDFILIAL'      =>$cdfilial, 
                'NRLANCESTQ'    =>$nrlancestq, 
                'NRORG'         =>$nrOrg,
                'IDEVOLESTOQ'   =>$idevolestoq
            );
            
            $std = $this->connection->prepare($sql);
            $update = $std->execute($params);
            return $update;
            
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function atualizaProdlcestoq($cdfilial, $cdalmoxarife, $cdlocalestoq, $cdproduto){
        try{
             $sql = "INSERT INTO PRODLCESTOQ
                    (CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO)
                    (SELECT :P_CDFILIAL, :P_CDALMOXARIFE, :P_CDLOCALESTOQ, :P_CDPRODUTO
                     FROM PRODFILI
                     WHERE CDFILIAL  = :P_CDFILIAL  AND
                           CDPRODUTO = :P_CDPRODUTO AND
                           NVL(IDCNTRESTOQ, 'S') <> 'N'
                     MINUS
                     SELECT CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO
                     FROM PRODLCESTOQ
                     WHERE CDFILIAL     = :P_CDFILIAL     AND
                           CDALMOXARIFE = :P_CDALMOXARIFE AND
                           CDLOCALESTOQ = :P_CDLOCALESTOQ AND
                           CDPRODUTO    = :P_CDPRODUTO)";
               
            $params = array("P_CDFILIAL" => $cdfilial,
                        "P_CDALMOXARIFE" => $cdalmoxarife,
                        "P_CDLOCALESTOQ" => $cdlocalestoq,
                        "P_CDPRODUTO"  => $cdproduto);
            
            $std = $this->connection->prepare($sql);
            $result = $std->execute($params);
            return $result;
            
        }catch (\Exception $e) {
            throw $e;
        }
    } 
    
    public function adqComposicaoLocal($row){
        try{
             $sql = "SELECT P.CDPRODUTO, P.CDARVPROD, R.QTSUBPRODLOC AS QTITEMCOMPOS,
                      R.NRHRFABPRLOC, R.NRSEQPRODLOC AS NRSEQUE, R.IDIMPPROTLOC, P.NMPRODUTO,
                      P.CDPRODESTO, P.SGUNIDADE, P.VRFATOCONV, PF.CDFILIAL,
                      PF.IDCNTRESTOQ, R.NRORG, P.CDPRODESTO
                    FROM ROTFABLOCAL R,
                      PRODUTO P,
                      PRODUTO O,
                      (SELECT COUNT(*) AS NRCOMPPRODL, CDPRODUTO FROM ROTFABLOCAL GROUP BY CDPRODUTO
                      ) F,  
                      PRODFILI PF
                    WHERE (O.CDPRODUTO  = :CDPRODUTO)
                    AND (O.CDPRODUTO    = R.CDPRODUTO)
                    AND (R.CDSUBPRODLOC = P.CDPRODUTO)
                    AND (F.CDPRODUTO(+) = R.CDSUBPRODLOC)
                    AND (PF.CDFILIAL    = :CDFILIAL)
                    AND (PF.CDPRODUTO   = P.CDPRODUTO)
                    AND (R.CDFILIAL     = :CDFILIAL)
                    ORDER BY NRSEQUE";
               
            $params = array("CDFILIAL" => $row['CDFILIAL'],
                            "CDPRODUTO" => $row['CDPRODUTO']);
            
            $std = $this->connection->prepare($sql);
            $result = $std->execute($params);
            return $std->fetchAll();
            
        }catch (\Exception $e) {
            throw $e;
        }
    } 
    
    public function adqComposicaoPadrao($row){
        try{
             $sql = "SELECT P.CDPRODUTO, P.CDARVPROD, R.QTSUBPRODROT AS QTITEMCOMPOS,
                      R.NRHRFABPROD, R.NRSEQPROD AS NRSEQUE, R.IDIMPPRODROT, P.NMPRODUTO,
                      P.CDPRODESTO, P.SGUNIDADE, P.VRFATOCONV, PF.CDFILIAL,
                      PF.IDCNTRESTOQ, R.NRORG, P.CDPRODESTO
                    FROM ROTFABRICA R,
                      PRODUTO P,
                      PRODUTO O,
                      (SELECT COUNT(*) AS NRCOMPPROD, CDPRODUTO FROM ROTFABRICA GROUP BY CDPRODUTO
                      ) F,  
                      PRODFILI PF
                    WHERE (O.CDPRODUTO  = :CDPRODUTO)
                    AND (O.CDPRODUTO    = R.CDPRODUTO)
                    AND (R.CDSUBPRODUTO = P.CDPRODUTO)
                    AND (F.CDPRODUTO(+) = R.CDSUBPRODUTO)
                    AND (PF.CDFILIAL    = :CDFILIAL)
                    AND (PF.CDPRODUTO   = P.CDPRODUTO)
                    ORDER BY NRSEQUE";
               
            $params = array("CDFILIAL" => $row['CDFILIAL'],
                        "CDPRODUTO" => $row['CDPRODUTO']);
            
            $std = $this->connection->prepare($sql);
            $result = $std->execute($params);
            return $std->fetchAll();
            
        }catch (\Exception $e) {
            throw $e;
        }
    } 
    
    public function adqProdutoEstoque($row){
        try{
             $sql = "SELECT P.CDPRODUTO, P.CDARVPROD, 1 AS QTITEMCOMPOS,
                      NULL AS NRHRFABPRLOC, NULL AS NRSEQUE, NULL AS IDIMPPROTLOC, P.NMPRODUTO,
                      P.CDPRODESTO, P.SGUNIDADE, P.VRFATOCONV, PF.CDFILIAL,
                      PF.IDCNTRESTOQ, P.NRORG, P.CDPRODESTO
                    FROM PRODUTO P,
                      PRODFILI PF
                    WHERE (P.CDPRODUTO  = :CDPRODUTO)
                    AND (PF.CDFILIAL    = :CDFILIAL)
                    AND (PF.CDPRODUTO   = P.CDPRODUTO)";
               
            $params = array("CDFILIAL" => $row['CDFILIAL'],
                           "CDPRODUTO" => $row['CDPRODUTO']);
            
            $std = $this->connection->prepare($sql);
            $result = $std->execute($params);
            return $std->fetchAll();
            
        }catch (\Exception $e) {
            throw $e;
        }
    } 
}
